Files
TheRock/rocm-libraries/projects/hipthreads/test/support/checking_mutex.h
T
derek.holloway 7502bf64a4
CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
Flatten subprojects into a single base monorepo
2026-09-02 16:34:47 -07:00

81 lines
2.2 KiB
C++

//===----------------------------------------------------------------------===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//
#ifndef TEST_SUPPORT_CHECKING_MUTEX_H
#define TEST_SUPPORT_CHECKING_MUTEX_H
#include <cassert>
#include <chrono>
struct checking_mutex {
enum state {
locked_via_lock,
locked_via_try_lock,
locked_via_try_lock_for,
locked_via_try_lock_until,
unlocked,
none,
};
state current_state = unlocked;
state last_try = none;
bool reject = false;
__host__ __device__ checking_mutex() = default;
__host__ __device__ checking_mutex(const checking_mutex&) = delete;
__host__ __device__ ~checking_mutex() { assert(current_state == unlocked); }
__host__ __device__ void lock() {
assert(current_state == unlocked);
assert(!reject);
current_state = locked_via_lock;
last_try = locked_via_lock;
reject = true;
}
__host__ __device__ void unlock() {
assert(current_state != unlocked && current_state != none);
last_try = unlocked;
current_state = unlocked;
reject = false;
}
__host__ __device__ bool try_lock() {
last_try = locked_via_try_lock;
if (reject)
return false;
current_state = locked_via_try_lock;
return true;
}
template <class Rep, class Period>
__host__ __device__ bool try_lock_for(const std::chrono::duration<Rep, Period>&) {
last_try = locked_via_try_lock_for;
if (reject)
return false;
current_state = locked_via_try_lock_for;
return true;
}
template <class Clock, class Duration>
__host__ __device__ bool try_lock_until(const std::chrono::time_point<Clock, Duration>&) {
last_try = locked_via_try_lock_until;
if (reject)
return false;
current_state = locked_via_try_lock_until;
return true;
}
checking_mutex* operator&() = delete;
template <class T>
void operator,(const T&) = delete;
};
#endif // TEST_SUPPORT_CHECKING_MUTEX_H